@@ -1,7 +1,5 @@
 from __future__ import absolute_import
-import difflib
 from functools import wraps, partial
-import re
 from flask import request, url_for, current_app
 from flask import abort as original_flask_abort
 from flask import make_response as original_flask_make_response
@@ -9,7 +7,6 @@ from flask.views import MethodView
 from flask.signals import got_request_exception
 from werkzeug.datastructures import Headers
 from werkzeug.exceptions import HTTPException, MethodNotAllowed, NotFound, NotAcceptable, InternalServerError
-from werkzeug.http import HTTP_STATUS_CODES
 from werkzeug.wrappers import Response as ResponseBase
 from flask_restful.utils import http_status_message, unpack, OrderedDict
 from flask_restful.representations.json import output_json
@@ -17,7 +14,10 @@ import sys
 from flask.helpers import _endpoint_from_view_func
 from types import MethodType
 import operator
-from collections import Mapping
+try:
+    from collections.abc import Mapping
+except ImportError:
+    from collections import Mapping
 
 
 __all__ = ('Api', 'Resource', 'marshal', 'marshal_with', 'marshal_with_field', 'abort')
@@ -50,8 +50,7 @@ class Api(object):
     after it has been constructed.
 
     :param app: the Flask application object
-    :type app: flask.Flask
-    :type app: flask.Blueprint
+    :type app: flask.Flask or flask.Blueprint
     :param prefix: Prefix all routes with a value, eg v1 or 2010-04-01
     :type prefix: str
     :param default_mediatype: The default media type to return
@@ -62,7 +61,7 @@ class Api(object):
         to handle 404 errors throughout your app
     :param serve_challenge_on_401: Whether to serve a challenge response to
         clients on receiving 401. This usually leads to a username/password
-        popup in web browers.
+        popup in web browsers.
     :param url_part_order: A string that controls the order that the pieces
         of the url are concatenated when the full url is constructed.  'b'
         is the blueprint (or blueprint registration) prefix, 'a' is the api
@@ -291,6 +290,13 @@ class Api(object):
 
         headers = Headers()
         if isinstance(e, HTTPException):
+            if e.response is not None:
+                # If HTTPException is initialized with a response, then return e.get_response().
+                # This prevents specified error response from being overridden.
+                # eg. HTTPException(response=Response("Hello World"))
+                resp = e.get_response()
+                return resp
+
             code = e.code
             default_data = {
                 'message': getattr(e, 'description', http_status_message(code))
@@ -312,30 +318,12 @@ class Api(object):
 
         data = getattr(e, 'data', default_data)
 
-        if code >= 500:
+        if code and code >= 500:
             exc_info = sys.exc_info()
             if exc_info[1] is None:
                 exc_info = None
             current_app.log_exception(exc_info)
 
-        help_on_404 = current_app.config.get("ERROR_404_HELP", True)
-        if code == 404 and help_on_404:
-            rules = dict([(re.sub('(<.*>)', '', rule.rule), rule.rule)
-                          for rule in current_app.url_map.iter_rules()])
-            close_matches = difflib.get_close_matches(request.path, rules.keys())
-            if close_matches:
-                # If we already have a message, add punctuation and continue it.
-                if "message" in data:
-                    data["message"] = data["message"].rstrip('.') + '. '
-                else:
-                    data["message"] = ""
-
-                data['message'] += 'You have requested this URI [' + request.path + \
-                                   '] but did you mean ' + \
-                                   ' or '.join((
-                                       rules[match] for match in close_matches)
-                                   ) + ' ?'
-
         error_cls_name = type(e).__name__
         if error_cls_name in self.errors:
             custom_data = self.errors.get(error_cls_name, {})
@@ -371,7 +359,7 @@ class Api(object):
         """Adds a resource to the api.
 
         :param resource: the class name of your resource
-        :type resource: :class:`Resource`
+        :type resource: :class:`Type[Resource]`
 
         :param urls: one or more url routes to match for the resource, standard
                      flask routing rules apply.  Any url variables will be
